/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package javax.tools.diagnostics.image;

/**
 * Represents a range of memory used for a specific purpose.
 * 
 * @see ImageModule
 */
public interface ImageSection {
    
    /**
     * Get the lowest address of memory in this section.
     * @return the lowest address of memory in this section
     */
    ImagePointer getBaseAddress();
    
    /**
     * Get the size of this contiguous image section as measured in bytes.
     * @return the size of this contiguous image section as measured in bytes
     */
    long getSize();
    
    /**
     * Get the name of this section (e.g. ".text").
     * @return the name of this section (e.g. ".text").  Note that sections of the image which have no specific 
     * name will receive a name synthesised by the implementation.  This will never be null.
     */
    String getName();
    
    /**
     * Does this section have permission to allow the processor to attempt to execute code?
     * @return true if this section is executable, false otherwise 
     * @throws DataUnavailable 
     */
    boolean isExecutable() throws DataUnavailable;
    
    /**
     * Is this section read-only ?
     * @return true if write access to this section was disabled
     * @throws DataUnavailable 
     */
    boolean isReadOnly() throws DataUnavailable;
    
    /**
     * Is this section shared with other processes..
     * @return true if this section is shared between processes
     * @throws DataUnavailable 
     */
    boolean isShared() throws DataUnavailable;
    
}
