/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package javax.tools.diagnostics.runtime;

import javax.tools.diagnostics.image.CorruptDataException;


/**
 * A generic managed runtime instance.
 * 
 * A Managed Runtime as against an "Unmanaged Runtime" is one where the runtime takes an active role in the program execution.
 * Common examples of managed runtimes are the Java Virtual Machine or the .NET Common Language Runtime.
 *  
 * <br>
 * No class should implement this interface directly. This is an marker interface which is extended by
 * specific runtime interfaces. See  {@link javax.tools.diagnostics.runtime.java.JavaRuntime} as an example of such a case.
 * 
 */
public interface ManagedRuntime {
    
    
    
	/**
	 * Returns version data available for this runtime instance. The version information is never null.
	 * The format of the version data is implementation specific. 
	 * 
	 * @return a string representing  the available version information specific to the implementation
	 * 
	 * @throws CorruptDataException If the ManagedRuntime implementation is unable to retrieve version data
	 */
	public String getVersion() throws CorruptDataException;

	/**
	 * @remove as redundant - a carryover from initial contribution
	 * 
	 * Get the 'full' version information for this runtime.
	 * @return a string representation of the version information for this runtime instance
	 * @throws CorruptDataException 
	 * @see #getVersion()
	 * 
	 * @deprecated Use "getVersion()" instead
	 */
	public String getFullVersion() throws CorruptDataException;    
}
