/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package javax.tools.diagnostics.runtime.java;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImagePointer;

/**
 * This class models the JavaVMOption C structures passed to the JNI invocation API entry point <a href="http://java.sun.com/javase/6/docs/technotes/guides/jni/spec/invocation.html#wp16334">JNI_CreateJavaVM</a> 
 * used to create a Java Virtual Machine.
 * 
 * Each JavaVMOption consists of two components	:
 * <ol>
 * <li> an optionString string, used to identify the option.</li>
 * <li> an extraInfo pointer, used to pass additional information. This component is usually null.</li>
 * </ol>
 */
public interface JavaVMOption {

	/**
	 * Fetch the optionString component of the option.
	 * @return a string representing the optionString. This is never null.
	 * @throws DataUnavailable
	 * @throws CorruptDataException
	 */
	String getOptionString() throws DataUnavailable, CorruptDataException;
	
	/**
	 * Fetch the extraInfo component of this option.
	 * @return the pointer value from the extraInfo (usually null).
	 * @throws DataUnavailable
	 * @throws CorruptDataException
	 */
	ImagePointer getExtraInfo() throws DataUnavailable, CorruptDataException;
}
