/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.jvmti.javaruntime.model;

import java.util.Collections;
import java.util.List;

import javax.tools.diagnostics.image.ImageSection;
import javax.tools.diagnostics.runtime.java.JavaHeap;
import javax.tools.diagnostics.runtime.java.JavaObject;

/**
 * Makes available all of the objects in the heap 
 * by extracting their List from the Model. 
 *
 */
public class JHeap implements JavaHeap {
	private static final String HEAP_NAME="Object Heap";	
	
	private Model model;
	
	public JHeap(Model model) {
		this.model = model;
	}

	/**
	 * Return a made-up name.
	 */
	@Override
	public String getName() {
		return HEAP_NAME;
	}

	@Override
	public List<JavaObject> getObjects() {
		return model.getObjects();
	}

	/**
	 * Returns empty list as we have no image sections to report. 
	 */
	@Override
	public List<ImageSection> getSections() {
		return Collections.emptyList();
	}

}
