/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.jvmti.javaruntime.model;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.logging.Level;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImageSection;
import javax.tools.diagnostics.runtime.java.JavaClass;
import javax.tools.diagnostics.runtime.java.JavaMethod;

import org.apache.kato.jvmti.reader.CLogger;

public class JMethod implements JavaMethod {

	private int argumentsize;
	public int maxlocals;
	public String name=null;
	public String signature=null;
	public int mods=0;
	public JClass parent=null;
	public HashMap<Long, Integer> lineNumberTable;
	private LinkedList<JLocalVariableTableEntry> localVars=null;
	public JMethod() {
	}
	
	public void setParent(JClass clazz) {
		parent=clazz;	
	}
	

	public void addLocalVariableTableEntry(JLocalVariableTableEntry entry) {
		if(localVars==null) {
			localVars=new LinkedList<JLocalVariableTableEntry>();
		}
		localVars.add(entry);
	}
	
	public void addLineNumberEntry(int lNum, long bytePos){
		if (lineNumberTable == null){
			lineNumberTable = new HashMap<Long, Integer>();
		}
		lineNumberTable.put(bytePos, lNum);
	}
	
	public int getLineNumber(long bytePos){
		if(lineNumberTable == null){
			return 0;
		}
		Vector<Long> v = new Vector<Long>(lineNumberTable.keySet());
		Collections.sort(v);
		Iterator<Long> it = v.iterator();
		while(it.hasNext()){
			long val = it.next();
			CLogger.logr.log(Level.FINEST,"Value "+bytePos+" index "+val);
			if(val>=bytePos){
				return lineNumberTable.get(val);
			}
		}
		return 0;
	}
	

	@Override
	public List<ImageSection> getBytecodeSections() {
		return Collections.emptyList();
	}

	@Override
	public List<ImageSection> getCompiledSections() {
		return Collections.emptyList();
	}

	@Override
	public JavaClass getDeclaringClass() throws CorruptDataException,
			DataUnavailable {		
		return parent;
	}

	@Override
	public int getModifiers() throws CorruptDataException {	
		return mods;
	}

	@Override
	public String getName() throws CorruptDataException {
		return name;
	}

	@Override
	public String getSignature() throws CorruptDataException {
		return signature;
	}

	@Override
	public List getVariables()  {
		if(localVars==null) {
			localVars=new LinkedList<JLocalVariableTableEntry>();
		}
		
		return localVars;
	}

	public JLocalVariableTableEntry getEntry(int slot, JLocation loc) {
		if(localVars==null) {
			localVars=new LinkedList<JLocalVariableTableEntry>();
		}
		for (JLocalVariableTableEntry jlvte: localVars){
			if (jlvte.getSlot() == slot){
				try {
					if (jlvte.start+jlvte.length >= loc.getAddress().getAddress() && jlvte.start <= loc.getAddress().getAddress()){
						return jlvte;
					}
				} catch (CorruptDataException e) {
				}
			}
		}
		for (JLocalVariableTableEntry jlvte: localVars){
			if (jlvte.getSlot() == slot){
				return jlvte;
			}
		}
		return null;
	}

	/** 
	 * Store generic signature for this method.
	 * Currently unimplemented as API doesn't require it.
	 *  
	 * @param signature
	 */
	public void setGenericsignature(String signature) {
		// Unimplemented
	}

	/**
	 * Returns the generic signature of this method.
	 * Currently not implemented.
	 * 
	 * @return null
	 */
	public String getGenericsignature() {
		return null;
	}
	
}