/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.jvmti.javaruntime.model;

import java.util.LinkedList;
import java.util.List;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.runtime.java.JavaMonitor;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaThread;

public class JMonitor implements JavaMonitor {
	private Model model;
	private long owner=0;
	private long monitorObject=0;
	private long id=0;
	private List<Long> waiters=new LinkedList<Long>();
	
	public JMonitor(long monitorid, Model model) {
		this.id = monitorid;
		this.model = model;
	}

	@Override
	public List<JavaThread> getEnterWaiters() {
		List<JavaThread> waitingThreads= new LinkedList<JavaThread>();
		for (Long threadID: waiters){
			JavaThread thread = model.getThread(threadID);
			
			try {
				if ((thread.getState() & JavaThread.STATE_IN_OBJECT_WAIT)  == 0) {
					waitingThreads.add(model.getThread(threadID));
				}
			} catch (CorruptDataException e) {
				e.printStackTrace();
			}
		}

		return waitingThreads;
	}

	public void addWaiter(Long t) {
		waiters.add(t);
	}

	@Override
	public ImagePointer getID() {
		// Possible issue
		return new SimpleImagePointer(0);
	}

	@Override
	public String getName() throws CorruptDataException {
		return "Monitor 0x"+Long.toHexString(getId());
	}

	@Override
	public List<JavaThread> getNotifyWaiters() {
		List<JavaThread> objectWaitThreads= new LinkedList<JavaThread>();
		for (Long threadID: waiters){
			JavaThread thread = model.getThread(threadID);
			
			try {
				if ((thread.getState() & JavaThread.STATE_IN_OBJECT_WAIT)  != 0) {
					objectWaitThreads.add(model.getThread(threadID));
				}
			} catch (CorruptDataException e) {
				e.printStackTrace();
			}
		}
		
		return objectWaitThreads;
	}

	@Override
	public JavaObject getObject() {
		return model.getObjectAtAddress(getMonitorObject());
	}

	@Override
	public JavaThread getOwner() throws CorruptDataException {
		// Return null if owner is 0 - otherwise a JThread with id 0 is created.
		return owner != 0 ? model.getThread(owner) : null;
	}
	
	public void setObject(long obj){
		this.monitorObject = obj;
	}

	public void setOwner(long thread) {
		this.owner=thread;
		
	}

	public long getId() {
		return id;
	}

	public long getMonitorObject() {
		return monitorObject;
	}
	
}