/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.jvmti.javaruntime.model;

import java.util.Collections;
import java.util.List;
import java.util.logging.Level;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.runtime.java.JavaLocation;
import javax.tools.diagnostics.runtime.java.JavaReference;
import javax.tools.diagnostics.runtime.java.JavaStackFrame;
import javax.tools.diagnostics.runtime.java.JavaVariable;

import org.apache.kato.jvmti.reader.CLogger;

public class JStackFrame implements JavaStackFrame {
	private static int stackFrameIDCount = 1;
	private JLocation location=null;
	private int stackFrameID;
	private ObjectMapList<Integer,JavaVariable> vars=new ObjectMapList<Integer, JavaVariable>();
	
	
	public JStackFrame(){
		stackFrameID = stackFrameIDCount;
		stackFrameIDCount++;
	}
	@Override
	public ImagePointer getBasePointer() throws CorruptDataException {
		
		return new SimpleImagePointer(stackFrameID);
	}

	public void setLocation(JLocation loc) {
		this.location=loc;
	}
	@Override
	public List<JavaReference> getHeapRoots() {
		return Collections.emptyList();
	}

	@Override
	public JavaLocation getLocation() throws CorruptDataException {
		
			return location;
	}

	@Override
	public Object getVariable(int slot) throws CorruptDataException,
			DataUnavailable, IndexOutOfBoundsException {
		if (vars == null){
			CLogger.logr.log(Level.FINEST,"Null variables "+slot);
			return null;
		}// TODO Correct this, behaviour is inconsistent
		JavaVariable jv =vars.get(slot);
		if (jv == null) {
			CLogger.logr.log(Level.FINEST,"Null variables in "+slot);
			return null;
		}
		Object r = jv.getValue();
		CLogger.logr.log(Level.FINEST,"asked for variable at slot "+slot+" ("+r+")");
		return r;
		
	}
	
	public void addVariable(JLocalVariable value) {
		value.setLocation(location);
		vars.put(value.slot, value);
	}

	@Override
	public List<JavaVariable> getVariables() {
		
		return vars.values();
	}

	public void addVariable(int slot, Object data) {
		JLocalVariable var=new JLocalVariable();
		var.slot=slot;
		var.value=data;
		addVariable(var);
		
	}
}
