/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.common;

import java.util.LinkedList;
import java.util.List;



public class BitMaskMappingArray {

	private ArrayBitMaskMappingStrategy strategy=null;
	private long[] data=null;
	private int    maxEntry=0;
	private IArrayEntryProvider provider=null;
	
	
	
	public BitMaskMappingArray(int entries,int bucketSize,IArrayEntryProvider provider) {
	
			strategy=new ArrayBitMaskMappingStrategy(entries,bucketSize);
			data=new long[entries];
			this.provider=provider;
	}
	
	/**
	 * Retrive element from index
	 * Returns null if element cannot be located
	 * 
	 * @param index 
	 * @return object at index or null
	 */
	public Object get(int index) {
		
		
//		List<String> state=new LinkedList<String>();
//		state.add("Get "+index);
//		provider.getState(state);
//		printState(state);
//		// ask strategy for element entry
		
		if(moveToLocation(index)==false) return null;
	
		
		return provider.getCurrentElement();
		
	}

	/**
	 * Return a record location at a particular location,
	 * rather than by index.
	 * 
	 * @param location within data provider
	 * @return Object
	 */
	public Object getAtLocation(long location) {
		if(provider.moveToLocation(location)==false) return null;
		
		return provider.getCurrentElement();
	}

	private boolean moveToLocation(int index) {
		
		int arraySlot=strategy.getIndexArrayElement(index);
		if(arraySlot<0) {
			
			// array too small
			// grow it...
			increaseCapacity(index);
			arraySlot=strategy.getIndexArrayElement(index);
		}
		
		// is target slot outside current known element?
		// if so fill array until slot reached
		if(arraySlot>maxEntry) {
			if(growArray(arraySlot)==false) return false;
		}
		
		// move provider to the required location
	
		if(provider.moveToLocation(data[arraySlot])==false) return false;
		
		int seekNo=strategy.getSeekCount(index);
		if(provider.moveRelativeElement(seekNo)==false) return false;
		
		return true;
	}

	private void printState(List<String> state) {
		int c=1;
		for(String s:state) {
			System.out.println(c+":"+s);
			c++;
		}
		
	}

	private boolean growArray(int arraySlot) {
	
		// move to last know slot
		long loc=data[maxEntry];
		provider.moveToLocation(loc);
		int bucketSize=strategy.getBucketSize();
		
		// slots to fill in 
		int slots=arraySlot-maxEntry;
		
		for(int i=0;i<slots;i++) {
			if(provider.moveRelativeElement(bucketSize)==false) return false;
			maxEntry++;
			data[maxEntry]=provider.getCurrentLocation();
		}
		return true;
	}

	private void increaseCapacity(int index) {
		
		do {
			maxEntry=ArrayCompactor.compact(data, maxEntry);
			strategy=strategy.doubleCapacity();
		}
		while (strategy.getIndexArrayElement(index)<0);
		
		
		
	}

	public long getLocation(int index) {
		if(moveToLocation(index)==false) return -1;
		
		return provider.getCurrentLocation();
	}
}
