/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.common;

import java.util.HashMap;
import java.util.Map;

/**
 * Maps from ID's to locations on the heap.
 * Uses a HashMap to do this in memory.
 * This will not scale, but will work as a trivial implementation.
 * 
 */
public class IDToLocationHashMapper implements IDLocationMap {	
	private Map<Long,Long>  idToLocation = new HashMap<Long,Long>();
		
	/**
	 * Retrieves a location by it ID.
	 * 
	 * @param ID
	 * @return
	 */
	public Long getLocation(Long ID) {
		Long location = idToLocation.get(ID);
		if (location == null) {
			return null;
		}
		
		return location;
	}
	
	/**
	 * Add a mapping from an ID to a location.
	 * 
	 * @param ID
	 * @param location
	 */
	public void put(Long ID, Long location) {
		if (idToLocation.containsKey(ID)) {
			System.err.println("Map already contains 0x"+Long.toHexString(ID));
		}
		idToLocation.put(ID, location);
	}
}
