/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.common;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

/**
 * Helper class to provide a List in circumstances where only an iterator is available
 * It builds a cache of objects that are used to satisfy requests to the 
 * list.  if the request cannot be satisfied then the provided iterator is 
 * used to add entries to the cache until the iterator is exhausted or the request
 * can be satisfied
 *  
 * The class honours the read-only semantics of the list interface.
 * 
 * @experimental This is an experimental addition and may be removed at a later date 
 *
 */
public class IteratorBackedList implements List {
	

	private LinkedList cache=new LinkedList();
	private Iterator fill=null;
	
	/**
	 * @param backing iterator
	 */
	public IteratorBackedList(Iterator f) {
		fill=f;
	}

	
	public boolean add(Object e) {
		
		return false;
	}

	public void add(int index, Object element) {
						
	}

	public boolean addAll(Collection c) {
	
		return false;
	}

	public boolean addAll(int index, Collection c) {

		return false;
	}

	public void clear() {
		
	}

	public boolean contains(Object o) {
		
		if(cache.contains(o)) return true;
		while(fill.hasNext()) {
			Object f=fill.next();
			cache.add(f);
			if(f==o) return true;
		}
		return false;
	
	}

	public boolean containsAll(Collection c) {
		fillCache();
		return cache.contains(c);
		
	}

	public Object get(int index) {
		int size=cache.size();
		if(index<size) return cache.get(index);
		
		while(fill.hasNext()) {
			Object f=fill.next();
			cache.add(f);
			size++;
			if(index<size) return cache.get(index);
		}
		
		return null;
	}

	public int indexOf(Object o) {
		
		int index=cache.indexOf(o);
		if(index>=0) return index;
		int size=cache.size();
		while(fill.hasNext()) {
			Object f=fill.next();
			cache.add(f);
			size++;
			if(f==o) return size-1;
		}
		return -1;
	}

	public boolean isEmpty() {
		if(cache.isEmpty()==false) return false;
		return fill.hasNext()==false;
	}

	public Iterator iterator() {
		fillCache();
		return cache.iterator();
	}

	public int lastIndexOf(Object o) {
		fillCache();
		return cache.lastIndexOf(o);
	}

	private void fillCache() {
		while(fill.hasNext()) {
			Object f=fill.next();
			cache.add(f);
		}
	}

	public ListIterator listIterator() {
		fillCache();
		return cache.listIterator();
	}

	public ListIterator listIterator(int index) {
		fillCache();
		return cache.listIterator(index);
	}

	public boolean remove(Object o) {
		return false;
	}

	public Object remove(int index) {
		return null;
	}

	public boolean removeAll(Collection c) {
		return false;
	}

	public boolean retainAll(Collection c) {
		return false;
	}

	public Object set(int index, Object element) {
		return null;
	}

	public int size() {
		fillCache();
		return cache.size();
	}

	public List subList(int fromIndex, int toIndex) {
		fillCache();
		return cache.subList(fromIndex, toIndex);
	}

	public Object[] toArray() {
		fillCache();
		return cache.toArray();
	}

	public Object[] toArray(Object[] a) {
		fillCache();
		return cache.toArray(a);

	}
}