<?xml version="1.0" encoding="utf-8"?>
<!--
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                version="1.0">
  <!-- import the main stylesheet, here pointing to fo/docbook.xsl -->
  <xsl:import href="urn:docbkx:stylesheet"/>
  <!-- highlight.xsl must be imported in order to enable highlighting support, highlightSource=1 parameter
   is not sufficient -->
  <xsl:import href="urn:docbkx:stylesheet/highlight.xsl"/>
 

  
  <xsl:attribute-set name="section.title.level1.properties">
    <xsl:attribute name="border-top">0.5pt solid black</xsl:attribute>
    <xsl:attribute name="border-bottom">0.5pt solid black</xsl:attribute>
    <xsl:attribute name="padding-top">6pt</xsl:attribute>
    <xsl:attribute name="padding-bottom">3pt</xsl:attribute>
  </xsl:attribute-set>
  
<xsl:attribute-set name="section.title.level1.properties">
  <xsl:attribute name="font-size">
    <xsl:value-of select="$body.font.master * 1.7"></xsl:value-of>
    <xsl:text>pt</xsl:text>
  </xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="section.title.level2.properties">
  <xsl:attribute name="border-top">0.5pt solid black</xsl:attribute>
  <xsl:attribute name="padding-top">6pt</xsl:attribute>
  <xsl:attribute name="font-size">
    <xsl:value-of select="$body.font.master * 1.5"></xsl:value-of>
    <xsl:text>pt</xsl:text>
  </xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="section.title.level3.properties">
  <xsl:attribute name="font-size">
    <xsl:value-of select="$body.font.master * 1.3"></xsl:value-of>
    <xsl:text>pt</xsl:text>
  </xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="section.title.level4.properties">
  <xsl:attribute name="font-size">
    <xsl:value-of select="$body.font.master"></xsl:value-of>
    <xsl:text>pt</xsl:text>
  </xsl:attribute>
</xsl:attribute-set>


<xsl:attribute-set name="figure.properties">
		<xsl:attribute name="color">#000000</xsl:attribute>
		<xsl:attribute name="font-weight">bold</xsl:attribute>
		<xsl:attribute name="background-color">#E0E0E0</xsl:attribute>
	</xsl:attribute-set>
<xsl:attribute-set name="example.properties">
		<xsl:attribute name="color">#000000</xsl:attribute>
		<xsl:attribute name="background-color">	#E0E0E0</xsl:attribute>
		<xsl:attribute name="font-size">8</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="monospace.properties">
  <xsl:attribute name="font-size">8</xsl:attribute>
</xsl:attribute-set>


<xsl:param name="page.margin.inner">1cm</xsl:param>
<xsl:param name="page.margin.outer">1cm</xsl:param>
<xsl:param name="highlight.source" select="1"></xsl:param>
<xsl:param name="highlight.default.language">java</xsl:param>


<xsl:attribute-set name="title.properties">
		<xsl:attribute name="color">#000000</xsl:attribute>
		<xsl:attribute name="font-weight">bold</xsl:attribute>
		<xsl:attribute name="background-color">#3366CC</xsl:attribute>
	</xsl:attribute-set>


	<xsl:attribute-set name="admonition.title.properties">
		<xsl:attribute name="font-size">
			<xsl:value-of select="$body.font.master * 1.5"/>
			<xsl:text>pt</xsl:text>
		</xsl:attribute>

		<xsl:attribute name="font-weight">bold</xsl:attribute>
		<xsl:attribute name="text-align">center</xsl:attribute>

		<xsl:attribute name="color">#FFFFFF</xsl:attribute>
		<xsl:attribute name="background-color">#17184A</xsl:attribute>

		<xsl:attribute name="border-color">#000000</xsl:attribute>
		<xsl:attribute name="border-style">solid</xsl:attribute>
		<xsl:attribute name="border-width">0px</xsl:attribute>
		<xsl:attribute name="padding-left">0.2cm</xsl:attribute>
		<xsl:attribute name="padding-right">0.2cm</xsl:attribute>
	</xsl:attribute-set>

	<!-- admonition (note/warn/caution) properties: gray background -->
	<xsl:attribute-set name="admonition.properties">
		<xsl:attribute name="background-color">#B0B3B2</xsl:attribute>
		<xsl:attribute name="border-color">#000000</xsl:attribute>
		<xsl:attribute name="border-style">solid</xsl:attribute>
		<xsl:attribute name="border-width">0px</xsl:attribute>
		<xsl:attribute name="padding-top">0.2cm</xsl:attribute>
		<xsl:attribute name="padding-bottom">0.2cm</xsl:attribute>
		<xsl:attribute name="padding-left">0.2cm</xsl:attribute>
		<xsl:attribute name="padding-right">0.2cm</xsl:attribute>
	</xsl:attribute-set>

	<xsl:attribute-set name="book.titlepage.recto.style">
		<xsl:attribute name="font-size">18px</xsl:attribute>
		<xsl:attribute name="text-align">center</xsl:attribute>
		<xsl:attribute name="padding-top">3cm</xsl:attribute>
	</xsl:attribute-set>
		
<!-- 
<xsl:template name="book.titlepage.recto">
   <fo:block>
        <fo:block font-size="28pt">JSR 326 User Manual and Specification</fo:block>
        <fo:block font-size="20pt">First Early Draft Review (2009)</fo:block>
        <fo:block space-before="5cm"> <fo:external-graphic src="url('figures/katologo.png')"
                       content-height="50em" content-width="50em"/>
  		</fo:block>
</fo:block>
 </xsl:template>
-->

</xsl:stylesheet>
