/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.hprof.datalayer;

import org.apache.kato.common.DataReader;

public abstract class HProfRecord implements IHProfRecord {

	private short tag=0;
	private byte[] data=null;
	public HProfRecord(short tag,byte[] data) {
		this.tag=tag;
		this.data=data;
	}

	/* (non-Javadoc)
	 * @see org.apache.kato.hprof.datalayer.IHProfRecord#getTag()
	 */
	public short getTag() {
		return tag;
	}

	
	protected int readInt(int offset) {
		return DataReader.read4Bytes(data,offset);
	}
	protected short readShort(int offset) {
		return DataReader.read2Bytes(data,offset);
	}
	
	protected long readNumber(int offset, int length) {
		return DataReader.readNumber(data,offset,length);
		
	}
	protected byte[] readBytes(int offset,int length) {
		return DataReader.readByteArray(data,offset,length);
	}
	
	/* (non-Javadoc)
	 * @see org.apache.kato.hprof.datalayer.IHProfRecord#getDataLength()
	 */
	public long getDataLength() {
		if(data==null) return 0;
		return data.length;
	}

	/* (non-Javadoc)
	 * @see org.apache.kato.hprof.datalayer.IHProfRecord#getData()
	 */
	public byte[] getData() {
		return data;
	}
}
