/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.hprof.datalayer;

/**
 * Represents a GCInstanceHeapDumpRecord.
 * This is a plain object instance, i.e. it is not an array.
 * 
 * 
 */
public interface IGCInstanceHeapDumpRecord extends IHeapObject {


	/**
	 * The object id of the class this object is an instance of.
	 * 
	 * @return long object id
	 */
	public abstract long getClassObjectID();


	/**
	 * Retrieve a boolean field from this object.
	 * 
	 * @param offset into instance values.
	 * @return byte value
	 */
	public abstract boolean getBooleanField(int offset);
	
	/**
	 * Retrieve a byte field from this object.
	 * 
	 * @param offset into instance values.
	 * @return byte value
	 */
	public abstract byte getByteField(int offset);
	
	/**
	 * Retrieve a char field from this object.
	 * 
	 * @param offset into instance values.
	 * @return char value
	 */
	public abstract char getCharField(int offset);
	
	/**
	 * Retrieve a short field from this object.
	 * 
	 * @param offset into instance values.
	 * @return short value
	 */
	public abstract short getShortField(int offset);
	
	/**
	 * Retrieve a int field from this object.
	 * 
	 * @param offset into instance values.
	 * @return int value
	 */
	public abstract int getIntegerField(int offset);
	
	/**
	 * Retrieve a long field from this object.
	 * 
	 * @param offset into instance values.
	 * @return long value
	 */
	public abstract long getLongField(int offset);
	
	/**
	 * Retrieve a float field from this object.
	 * 
	 * @param offset into instance values.
	 * @return float value
	 */
	public abstract float getFloatField(int offset);
	
	/**
	 * Retrieve a double field from this object.
	 * 
	 * @param offset into instance values.
	 * @return double value
	 */
	public abstract double getDoubleField(int offset);
	
	/**
	 * Retrieve an object ID field from this object.
	 * 
	 * @param offset into instance values.
	 * @return long value - an object ID.
	 */
	public abstract long getIDField(int offset);
}
