/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tools.jdi;

import java.util.Vector;

public class ReplyPacket extends Packet {
	private static final long serialVersionUID = 1L;
	private int nErrorCode;
	
	public ReplyPacket(int seq, int flags, int errorcode, JDISocket ivsock){
		nSeq = seq;
		nFlags = flags;
		nErrorCode = errorcode;
		setVSock(ivsock);
	}
	
	public ReplyPacket(int seq, int flags, int errorcode){
		nSeq = seq;
		nFlags = flags;
		nErrorCode = errorcode;
	}
	
	public void setSequence(int seq){
		nSeq = seq;
	}
	
	public int getErrorCode(){
		return nErrorCode;
	}
	
	@Override
	public byte[] toBytes(){
		Vector<Byte> vctr = new Vector<Byte>();
		int len = 11 + getByteData().length;
		KatoReader.addIntToVector(vctr, len);
		KatoReader.addIntToVector(vctr, nSeq);
		vctr.add((byte)nFlags);
		vctr.add((byte)((nErrorCode >> 8)&0xFF));
		vctr.add((byte)(nErrorCode & 0xFF));
		if (len > 11){
			byte[] outData = getByteData();
			for (int i = 0; i < outData.length; i++){
				vctr.add(outData[i]);
			}
		}
		
		return KatoReader.vectorToByte(vctr);
		
		
	}
	
	
}
