/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/


#include "cjvmti.h"
#include "addClass.h"
#include "addField.h"
#include "addObject.h"


int addMethodDetails(jclass class);

struct JClass * addClassDetails(jclass class) {
	jint field_count;
	jvmtiError err;
	jfieldID * fields;
	char * name_ptr = NULL;
	char * signature_ptr = NULL;
	char * genSig_ptr = NULL;
	char * sourceFileName = NULL;
	jlong tag;
	struct JClass * jcls = NULL;
	struct JObject * oldObj = NULL;
	int i;
	int staticFieldCount = 0;
	int instanceFieldCount = 0;
	int countUp;
	int classObject;
	jint modifiers;
	jint statusP;

	jint interfaceCount;
	jclass * interfaces = NULL;
	struct JClass * interfaceClass = NULL;

	jint classMods;


	FPOS_T fieldPos;
	FPOS_T fieldPosCheck;


	int level = 0;
	int subLevel = 0;
	jclass superClazz;

	printDepth++;
	if (class == 0) {
		return NULL;
	}
	statusP = 0;
	// Make a call to check for errors on this class
	err = (*env)->GetClassStatus(env, class, &statusP);
	if (err != JVMTI_ERROR_NONE){
		printf("Class is not loaded \n");
		return NULL;
	}
	
	// Check class is valid before attempting to save any information about it.
	(*env)->GetClassModifiers(env, class, &classMods);
	//if (statusP == 0 || statusP & 8){
	if (!(statusP == 7 || statusP == 16 || statusP == 32 || statusP == 3 )){
		if (statusP & 8){
			printf("Invalid class: loading error \n");}
		return NULL;
	}
	
	err = (*env)->GetTag(env, class, &tag);
	if (err != JVMTI_ERROR_NONE) {
		return NULL;
	}

	// Check if this has already been added, but as an object, not a class object.
	classObject = 0;
	if (tag!=0){
		oldObj = (void *)tag;
		if (oldObj->type == OBJECT){
			classObject = 1;
			oldObj->type = CLASS;
		}
	}

	

	
	if (tag == 0 || classObject) {
		FSEEK(variableFile, 0, SEEK_END);

		// Follow class hierarchy from the bottom up, reduces recursion which isn't currently prevented on classes, only object

		// Calculate bottom depth
		superClazz = class;
		while (superClazz != 0) {
			level++;
			superClazz = (*jniptr)->GetSuperclass(jniptr, superClazz);
		}

		// Work back up adding the class details.
		superClazz = class;
		while (level != 0) {
			superClazz = class;
			subLevel = level;
			while (subLevel != 0) {
				superClazz = (*jniptr)->GetSuperclass(jniptr, superClazz);
				subLevel--;
			}
			level--;
			if (superClazz == 0) {
			} else {
				addClassDetails(superClazz);
			}
			FSEEK(variableFile, 0, SEEK_END);
		}


		addClassDetails((*jniptr)->GetSuperclass(jniptr, class)); // Ensure we know everything about 
		//this class hierarchy before beginning, should return immediately

		// Begin writing class details
		FSEEK(variableFile, 0, SEEK_END);
		err = (*env)->GetClassSignature(env, class, &name_ptr, &genSig_ptr);
		if (err != JVMTI_ERROR_NONE){
			(*env)->Deallocate(env, (unsigned char*)genSig_ptr);
			(*env)->Deallocate(env, (unsigned char*)name_ptr);
			printf(" Unexpected JVMTI error %d \n", err);
			abort();
		}

		tag = (long) (struct JClass *) calloc(1, sizeof(struct JClass));

		if (tag == (long) NULL) {
			printf("Failed to malloc class \n");
			abort();
		}
		
		err = (*env)->SetTag(env, class, tag);

		Q_enqueue(&class, toDetag); // Add to deallocate queue

		jcls = (void *) tag;

		if (classObject){ // If this was previously just an object copy object information and deallocate old object.
			jcls->obj = (*oldObj);
			free(oldObj);
		}
		jcls->positionInFile = FTELL(variableFile);
		jcls->obj.type = CLASS; // Indicates its a class object
		staticsCount++;
		
		writeIDNoSize(CJVMTI_CLASS, variableFile);
		writeJint(classMods, variableFile);
		writeString(name_ptr, variableFile);
		sourceFileName = 0;
		err = (*env)->GetSourceFileName(env, class, &sourceFileName);
		if (err != JVMTI_ERROR_NONE || sourceFileName == 0) {
			(*env)->Deallocate(env, (unsigned char*)sourceFileName);
			writeString("NoSource", variableFile);
		} else {
			writeString(sourceFileName, variableFile);
			(*env)->Deallocate(env, (unsigned char*)sourceFileName);
		}

		if (genSig_ptr != 0){
			writeString(genSig_ptr, variableFile);
			(*env)->Deallocate(env, (unsigned char*)genSig_ptr);
		}else {
			(*env)->Deallocate(env, (unsigned char*)genSig_ptr);
			writeString(" ", variableFile);
		}
	} else {
		jcls = (void *) tag; // Already disocvered so return class.
		return jcls;
	}

	err = (*env)->GetClassFields(env, class, &field_count, &fields);
	if (err != JVMTI_ERROR_NONE) {
		jcls->numInstanceFields = 0;
		printf("Unexpected JVMTI error %d \n", err);
		abort();
	}

	addMethodDetails(class);

	jcls->name = name_ptr;
	name_ptr = NULL;

	// get field type counts
	for (i = 0; i < field_count; i++) {
		err = (*env)->GetFieldModifiers(env, class, fields[i], &modifiers);
		assert(err == JVMTI_ERROR_NONE);
		if (modifiers & 0x0008) {
			staticFieldCount++;
		} else {
			instanceFieldCount++;
		}
	}
	jcls->numStaticFields = staticFieldCount;

	// malloc space for them..
	jcls->numInstanceFields = instanceFieldCount;
	jcls->instanceModifiers
			= (jint *) malloc(sizeof(jint) * instanceFieldCount);
	jcls->instanceFieldGenSig = (char * *) malloc(sizeof(char *)
			* instanceFieldCount);
	jcls->instanceFieldSignatures = (char * *) malloc(sizeof(char *)
			* instanceFieldCount);
	jcls->instanceFields = (jfieldID *) malloc(sizeof(jfieldID)
			* instanceFieldCount);

	if (jcls->instanceFields == NULL) {
		printf("Failed to malloc instance fields\n");
		abort();
	}

	writeIDNoSize(CJVMTI_CLASS_INSTANCE_FIELDS, variableFile);
	writeJint(instanceFieldCount, variableFile);

	// occupy struct with instance fields so we can save objects correctly
	countUp = 0;
	for (i = 0; i < field_count; i++) {
		(*env)->GetFieldModifiers(env, class, fields[i], &modifiers);
		if (!(modifiers & 0x0008)) {
			err = (*env)->GetFieldName(env, class, fields[i], &name_ptr,
					&signature_ptr, &genSig_ptr);
			if (err != JVMTI_ERROR_NONE) {
				printf("Error getting field name %d \n", err);
				abort();
			}

			writeJint(modifiers, variableFile);
			writeReference((FPOS_T) fields[i], variableFile);
			writeString(name_ptr, variableFile);
			writeString(signature_ptr, variableFile);
			if (genSig_ptr == 0) {
				writeString(" ", variableFile);
			} else {
				writeString(genSig_ptr, variableFile);
			}

			instanceFieldCount--;
			assert(instanceFieldCount > -1);
			jcls->instanceFields[countUp] = fields[i];
			jcls->instanceFieldSignatures[countUp] = signature_ptr;
			if ((genSig_ptr) != 0) {
				jcls->instanceFieldGenSig[countUp] = genSig_ptr;
			} else {
				jcls->instanceFieldGenSig[countUp] = 0;
			}
			jcls->instanceModifiers[countUp] = modifiers;
			countUp++;

			(*env)->Deallocate(env, (unsigned char*)name_ptr);
			name_ptr = NULL;
		}
	}

	// write out static field details, these only need saving to the dump file.
	writeIDNoSize(CJVMTI_CLASS_STATIC_FIELDS, variableFile);
	writeJint(staticFieldCount, variableFile);
	genSig_ptr = NULL;
	for (i = 0; i < field_count; i++) {
		(*env)->GetFieldModifiers(env, class, fields[i], &modifiers);
		if ((modifiers & 0x0008)) {
			err = (*env)->GetFieldName(env, class, fields[i], &name_ptr,
					&signature_ptr, &genSig_ptr);
			if (err != JVMTI_ERROR_NONE) {
				break;
			}

			writeJint(modifiers, variableFile);
			writeReference((FPOS_T) fields[i], variableFile);
			writeString(name_ptr, variableFile);
			writeString(signature_ptr, variableFile);
			if (genSig_ptr == 0) {
				writeString(" ", variableFile);
				(*env)->Deallocate(env, (unsigned char*)genSig_ptr);
			} else {
				writeString(genSig_ptr, variableFile);
				(*env)->Deallocate(env, (unsigned char*)genSig_ptr);
			}
			(*env)->Deallocate(env, (unsigned char*)name_ptr);
			(*env)->Deallocate(env, (unsigned char*)signature_ptr);
		}
	}


	// make space for all references in file.

	fieldPos = FTELL(variableFile);
	writeIDNoSize(CJVMTI_SUPERCLASS_FIELDS, variableFile);
	fwrite(&waitingToBeWritten, sizeof(FPOS_T), 1, variableFile); // super class reference

	// Work out space needed for interface references
	err = (*env)->GetImplementedInterfaces(env, class, &interfaceCount, &interfaces);
	if (err != JVMTI_ERROR_NONE){
		interfaceCount = 0;
	}
	writeIDNoSize(CJVMTI_CLASS_INTERFACES, variableFile);
	writeJint(interfaceCount, variableFile);
	for (i = 0; i < interfaceCount; i++) {
		fwrite(&waitingToBeWritten, sizeof(FPOS_T), 1, variableFile);
	}

	writeIDNoSize(CJVMTI_CLASS_STATIC_FIELDS, variableFile);
	for (i = 0; i < staticFieldCount; i++) {
		fwrite(&waitingToBeWritten, sizeof(FPOS_T), 1, variableFile);
	}
	

	writeIDNoSize(CJVMTI_CLASSLOADERS, variableFile);
	fwrite(&waitingToBeWritten, sizeof(FPOS_T), 1, variableFile); // class loader reference


	FGETPOS(variableFile, &fieldPosCheck); //Alignment check for end
	
	// Follow references

	FSEEK(variableFile, 0, SEEK_END);
	jcls->superClass = addClassDetails((*jniptr)->GetSuperclass(jniptr, class));
	FSETPOS(variableFile, &fieldPos);

	writeIDNoSize(CJVMTI_SUPERCLASS_FIELDS, variableFile);
	if (jcls->superClass == 0) {
		writeReference(CJVMTI_NULL_OBJECT, variableFile);
	} else {
		writeReference(jcls->superClass->positionInFile, variableFile);
	}

	writeIDNoSize(CJVMTI_CLASS_INTERFACES, variableFile);
	writeJint(interfaceCount, variableFile);
	fieldPos = FTELL(variableFile);
	FSEEK(variableFile, 0, SEEK_END);
	for (i = 0; i < interfaceCount; i++) {
		interfaceClass = addClassDetails(interfaces[i]);
		FSETPOS(variableFile, &fieldPos);
		writeReference(interfaceClass->positionInFile, variableFile);
		fieldPos = FTELL(variableFile);
		FSEEK(variableFile, 0, SEEK_END);
	}
	(*env)->Deallocate(env, (unsigned char*)interfaces);
	(*env)->Deallocate(env, (unsigned char*)fields);

	jcls->staticFieldStart = fieldPos;
	FSEEK(variableFile, 0, SEEK_END);
	Q_enqueue(&class, qStaticFields); // Add to class queue to resolve static field references
	return jcls;
}

// Resolves static field references
struct JClass * getClassReferences(jclass class) {
	FPOS_T fieldPos;
	FPOS_T reference;
	jfieldID * fields = NULL;
	jint field_count;
	int i;
	jint err;
	char * name_ptr = NULL;
	char * signature_ptr = NULL;
	char * genSig_ptr = NULL;
	jint modifiers;
	int staticFieldCount;
	jobject classloader;

	struct JClass * jcls = addClassDetails(class);
	if (jcls == NULL) {
		return NULL;
	}
	if (jcls->filledIn) {// Indicates references have already been resolved
		return jcls;
	}

	staticFieldCount = jcls->numStaticFields;
	FSEEK(variableFile, jcls->staticFieldStart, SEEK_SET);

	writeIDNoSize(CJVMTI_CLASS_STATIC_FIELDS, variableFile);
	fieldPos = FTELL(variableFile);
	FSEEK(variableFile, 0, SEEK_END);
	err = (*env)->GetClassFields(env, class, &field_count, &fields);
	
	// Follow each reference, then save
	for (i = 0; i < field_count; i++) {
		err = (*env)->GetFieldName(env, class, fields[i], &name_ptr,
				&signature_ptr, &genSig_ptr);
		if (err != JVMTI_ERROR_NONE) {
			printf("Getting field name for class error %d\n", err);
			abort();
			break;
		}
		// check if field is static
		(*env)->GetFieldModifiers(env, class, fields[i], &modifiers);
		if (modifiers & 0x0008) {
			staticFieldCount--;
			reference = 0;
			reference = addFieldValue(CJVMTI_STATIC_VAR, class, fields[i],
							signature_ptr, 0, 0, 0, 0);
			FSETPOS(variableFile, &fieldPos);
			writeReference(reference, variableFile);

			// check its added a true reference
			if (reference == waitingToBeWritten) {
				printf("Circular reference, abort \n");
				abort();
			}
			fieldPos = FTELL(variableFile);
			FSEEK(variableFile, 0, SEEK_END);
		}

		(*env)->Deallocate(env, (unsigned char *) name_ptr);
		(*env)->Deallocate(env, (unsigned char *) signature_ptr);
		(*env)->Deallocate(env, (unsigned char *) genSig_ptr);
		name_ptr = NULL;
		signature_ptr = NULL;
		genSig_ptr = NULL;
	}

	// write class loader details to file
	
	FSEEK(variableFile, 0, SEEK_END);

	jcls->filledIn = 1;
	(*env)->GetClassLoader(env, class, &classloader);
    {
        struct JClassLoader *jcl = getClassLoaderDetails(classloader);
        jcls->classLoader = (jcl == NULL) ? NULL : jcl->positionInFile;
    }

	FSETPOS(variableFile, &fieldPos);

	writeIDNoSize(CJVMTI_CLASSLOADERS, variableFile);
	writeReference(jcls->classLoader, variableFile);
	fieldPos = FTELL(variableFile);
	FSEEK(variableFile, 0, SEEK_END);


	printDepth--;
	staticsCount--;
	(*env)->Deallocate(env, (unsigned char *)fields);
	return jcls;
}







int addMethodDetails(jclass class) {
	jint method_count;
	jmethodID * method_ptr = NULL;
	jvmtiError err;
	char * mname = NULL;
	char * msig = NULL;
	char * mgenSig = NULL;
	jint varCount;
	jvmtiLocalVariableEntry * table = NULL;
	jvmtiLineNumberEntry * lnt_ptr = NULL;
	jint modifiers;
	int i;
	int i2;

	writeIDNoSize(CJVMTI_METHOD, variableFile);
	err = (*env)->GetClassMethods(env, class, &method_count, &method_ptr);
	if (err != JVMTI_ERROR_NONE) {
		(*env)->Deallocate(env, (unsigned char *) method_ptr);
		method_count = 0;
	}
	writeJint(method_count, variableFile);
	for (i = 0; i < method_count; i++) {
		writeIDNoSize(CJVMTI_METHOD, variableFile);
		err
				= (*env)->GetMethodName(env, method_ptr[i], &mname, &msig,
						&mgenSig);
		if (err != JVMTI_ERROR_NONE) {
			mname = "Missing";
			continue;
		}
		writeReference((FPOS_T) method_ptr[i], variableFile);
		writeString(mname, variableFile);
		writeString(msig, variableFile);
		(*env)->Deallocate(env, (unsigned char *) mname);
		(*env)->Deallocate(env, (unsigned char *) msig);
		if (mgenSig != 0) {
			writeString(mgenSig, variableFile);
			(*env)->Deallocate(env, (unsigned char *) mgenSig);
		} else {
			writeString(" ", variableFile);
			(*env)->Deallocate(env, (unsigned char *) mgenSig);
		}
		mname = NULL;
		msig = NULL;
		mgenSig = NULL;

		err = (*env)->GetMethodModifiers(env, method_ptr[i], &modifiers);
		writeJint(modifiers, variableFile);
		err = (*env)->GetLocalVariableTable(env, method_ptr[i], &varCount,
				&table);
		if (err != JVMTI_ERROR_NONE) {
			varCount = 0;
			writeJint(0, variableFile);
		} else {
			writeJint(varCount, variableFile);
		}
		for (i2 = 0; i2 < varCount; i2++) {
			writeString(table[i2].name, variableFile);
			writeString(table[i2].signature, variableFile);
			(*env)->Deallocate(env, (unsigned char *) table[i2].name);
			(*env)->Deallocate(env, (unsigned char *) table[i2].signature);
			if (table[i2].generic_signature != 0) {
				writeString(table[i2].generic_signature, variableFile);
				(*env)->Deallocate(env, (unsigned char *) table[i2].generic_signature);
			} else {
				writeString(" ", variableFile);
			}
			writeJint(table[i2].length, variableFile);
			writeJint(table[i2].slot, variableFile);
			writeReference(table[i2].start_location, variableFile);
			
			
		}
		(*env)->Deallocate(env, (unsigned char *) table);
		table = NULL;

		// Line number table
		(*env)->GetLineNumberTable(env, method_ptr[i], &varCount, &lnt_ptr);
		if (err != JVMTI_ERROR_NONE) {
			writeJint(0, variableFile);
		} else {
			writeJint(varCount, variableFile);
			for (i2 = 0; i2 < varCount; i2++) {
				writeJint(lnt_ptr[i2].line_number, variableFile);
				writeReference(lnt_ptr[i2].start_location, variableFile);
			}
			
		}
		(*env)->Deallocate(env, (unsigned char *) lnt_ptr);
		lnt_ptr = NULL;

	}

	(*env)->Deallocate(env, (unsigned char *) method_ptr);
	return 1;
}


/*
 * Adds classloader details to the file.
 * Each class calls this function with it's classloader object.
 * The classloader stores the object reference, the number of classes
 * and a reference to each of the classes.
 * This might have to transform the java object tag into a classloader.
 *
 * Format:
 * CJVMTI_CLASSLOADER | OBJECT REFERENCE | jint numberClasses | jlong classloader | ...
 */
struct JClassLoader * getClassLoaderDetails(jobject object) {
    jvmtiError err;
    struct JObject *oldObj;
    jlong tag;
    struct JClassLoader *jcl;
    char isObject = 0;

    if (object != NULL) {
    	err = (*env)->GetTag(env, object,(jlong *) &tag);

        if ((tag != NULL) && (((struct JObject*) tag)->type == OBJECT)) {
            isObject = 1; 
            oldObj = (struct JObject *) tag;
        }
	    if (err != JVMTI_ERROR_NONE) {
		    return NULL;
    	}
    } else {
        tag = (jlong) systemLoader;
    }

    if ((tag == NULL) || (isObject == 1)) {
        jint class_count; /* The number of classes in this classloader.*/
        jclass *classes; /* The array of classes in this classloader. */
        jint i;
        FPOS_T objPos, classesPos;

        jcl = (struct JClassLoader *) calloc(1, sizeof(struct JClassLoader));
        tag = (jlong) jcl;

        if (tag == NULL) {
            fprintf(stderr, "Failed to malloc JClassLoader structure.");
            abort();
        }

        if (isObject) {
            memcpy(jcl, oldObj, sizeof(struct JObject));
            jcl->obj.type = CLASSLOADER;
            free(oldObj);
        }

        err = (*env)->GetClassLoaderClasses(env, object, &class_count, &classes);

        if ((classes == NULL) && (class_count != 0)) {
            fprintf(stderr, "GetClassloaderClasses returns a NULL class array.\n", err);
            abort();
        }

        if (err != JVMTI_ERROR_NONE) {
            fprintf(stderr, "Unexpected JVMTI error %d in GetClassloaderClasses\n", err);
            abort();
        }


        /* If not the bootstrap classloader, get the Java object
         * (ClassLoader instance) and store it's position,
         * setup the tag for deallocation and setup tag for
         * deallocation.
         */
        if (object != NULL) {
            err = (*env)->SetTag(env, object, (jlong) tag);
            Q_enqueue(&object, toDetag);
            objPos = addObject(object, 0)->positionInFile;
        } else {
            systemLoader = jcl;
            objPos = CJVMTI_NULL_OBJECT;
        }


        /* Store object ID for later when classes are resolved.
         */
        jcl->objID = object; 
    
        /* Set the tag:
         * CJVMTI_CLASSLOADER | OBJECT REFERENCE | jint numberClasses
         * Remember position in tag.
         */
        FSEEK(variableFile, 0, SEEK_END);
        jcl->positionInFile = FTELL(variableFile);
        writeIDNoSize(CJVMTI_CLASSLOADER, variableFile);
        writeReference((FPOS_T) objPos, variableFile);
        writeJint(class_count, variableFile);
        
        /* Reserve space for class references */
        for(i=0; i < class_count; i++) {
            writeReference(CJVMTI_NULL_OBJECT, variableFile);
        }        

        /* Ensure class details exist to be written */
        for(i=0; i < class_count; i++) {
            addClassDetails(classes[i]);
        }

        if (classes != NULL) { 
            (*env)->Deallocate(env, (unsigned char*) classes);
        }

        Q_enqueue(&jcl, qClassLoaders);

        FSEEK(variableFile, 0, SEEK_END);
        return jcl;
    }

    return (struct JClassLoader *) tag;
}


/*
 * Fill in references to classes from classloader
 * This is executed after all other records have been written.
 * 
 */
void writeClassLoader(struct JClassLoader *loader) {
    jvmtiError err;
    jint class_count; /* The number of classes in this classloader.*/
    jclass *classes; /* The array of classes in this classloader. */
    jint i;
    FSEEK(variableFile, loader->positionInFile+1+8+4, SEEK_SET);

    
    err = (*env)->GetClassLoaderClasses(env, loader->objID, &class_count, &classes);

        if ((classes == NULL) && (class_count != 0)) {
            fprintf(stderr, "GetClassloaderClasses returns a NULL class array.\n", err);
            abort();
        }

        if (err != JVMTI_ERROR_NONE) {
            fprintf(stderr, "Unexpected JVMTI error %d in GetClassloaderClasses\n", err);
            abort();
        }

        /* Write out all references to the classes */
        for(i=0; i < class_count; i++) {
            jlong clTag;

	        err = (*env)->GetTag(env, classes[i], (jlong *)&clTag);

            
            if (err != JVMTI_ERROR_NONE) {
                fprintf(stderr, "Error getting tag from class %p, got error %d\n", classes[i], err); 
                abort();
            } else if (clTag == NULL) {
                /* The class might not be in a fit state to be stored. */
                writeReference( CJVMTI_NULL_OBJECT, variableFile);
            } else {
                /* The classes positions should all be resolve by now. Write them out. 
                 * but only if they are classes - some might still exist as unresolved objects.
                 */
                if(((struct JObject *)clTag)->type == CLASS) {
                    writeReference((FPOS_T) ((struct JClass *)clTag)->positionInFile, variableFile);
                } else {
                    writeReference( CJVMTI_NULL_OBJECT, variableFile);
                }
                /* Ensure we don't accumulate the class references */
                (*jniptr)->DeleteLocalRef(jniptr, classes[i]);
            }

        } 

        if (classes != NULL) { 
            (*env)->Deallocate(env, (unsigned char*) classes);
        }
        
        if(loader->objID != NULL) {
            (*env)->SetTag(env, loader->objID, NULL);
        }        
        free(loader);
}
