/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/


#include "cjvmti.h"
#include "addObject.h"
#include "addField.h"
#include "addClass.h"

int getLocVars(jmethodID methodID, jlocation loc, jthread * thread, int depth) {
	int i = 0;
	int aliveVarCount;
	jvmtiError err;
	jint varCount;
	jvmtiLocalVariableEntry * table = NULL;
	FPOS_T refLoc;
	FPOS_T tempLoc;
	aliveVarCount = 0;
	printDepth++;
	
	if (loc == -1) {
		printDepth--;
		writeIDNoSize(CJVMTI_LOCAL_NATIVECALL, variableFile);
		return 0;
	}
	err = (*env)->GetLocalVariableTable(env, methodID, &varCount, &table);
	if (err != JVMTI_ERROR_NONE) {
		writeIDNoSize(CJVMTI_JVMTI_ERROR, variableFile);
		printDepth--;
		(*env)->Deallocate(env, (unsigned char*)table);
		return 0;
	}
	writeIDNoSize(CJVMTI_LOCAL_VARIABLE, variableFile);
	fwrite(&loc, sizeof(jlocation), 1, variableFile);

	// Work out how many are currently available
	for (i = 0; i < varCount; i++) {
		if ((table[i].start_location <= loc) && ((table[i].start_location
				+ table[i].length) >= loc)) {
			aliveVarCount++;
		}
	}
	writeJint(aliveVarCount, variableFile);
	refLoc = FTELL(variableFile);
	for (i = 0; i < aliveVarCount; i++) {
		writeJint(0, variableFile);
		writeReference(waitingToBeWritten, variableFile);
	}

	// Follow the available slots and output data.
	FSEEK(variableFile, refLoc, SEEK_SET);
	for (i = 0; i < varCount; i++) {
		if ((table[i].start_location <= loc) && ((table[i].start_location
				+ table[i].length) >= loc)) {
			writeJint(table[i].slot, variableFile);
			refLoc = FTELL(variableFile);
			FSEEK(variableFile, 0, SEEK_END);
			tempLoc = addFieldValue(CJVMTI_LOCAL_VAR, NULL, NULL,
					table[i].signature, *thread, depth, table[i].slot, 0);
			FSEEK(variableFile, refLoc, SEEK_SET);
			writeReference(tempLoc, variableFile);
			(*env)->Deallocate(env, (unsigned char*)table[i].generic_signature);
			(*env)->Deallocate(env, (unsigned char*)table[i].name);
			(*env)->Deallocate(env, (unsigned char*)table[i].signature);
		} else {
			(*env)->Deallocate(env, (unsigned char*)table[i].generic_signature);
			(*env)->Deallocate(env, (unsigned char*)table[i].name);
			(*env)->Deallocate(env, (unsigned char*)table[i].signature);
		}
	}
	(*env)->Deallocate(env, (unsigned char*)table);
	printDepth--;
	return 1;
}

FPOS_T getFrameInfo(int depth, jvmtiFrameInfo * info, jthread * thread) {
	jvmtiError err;
	jclass declClass;
	FPOS_T dClassPos;
	FPOS_T position;

	char* methodName = NULL;
	struct JClass * declClazz;
	printDepth++;
	err = (*env)->GetMethodName(env, info->method, &methodName, NULL, NULL);
	err = (*env)->GetMethodDeclaringClass(env, info->method, &declClass);
	if (err != JVMTI_ERROR_NONE) {
		position = FTELL(variableFile);
		writeIDNoSize(CJVMTI_JVMTI_ERROR, variableFile);
		(*env)->Deallocate(env, (unsigned char*)methodName);
		printf("Unexpected JVMTI error %d \n", err);
		abort();
	}
	FSEEK(variableFile, 0, SEEK_END);

	// Add declaring class reference
	if (declClass != 0) {
		declClazz = addClassDetails(declClass);
		if (declClazz != NULL){
			dClassPos = declClazz->positionInFile;}else
			{
				dClassPos = CJVMTI_NULL_OBJECT;
			}
	} else {
		dClassPos = CJVMTI_NULL_OBJECT;
	}
	FSEEK(variableFile, 0, SEEK_END);
	position = FTELL(variableFile);
	writeIDNoSize(CJVMTI_FRAME, variableFile);
	writeReference((FPOS_T) info->method, variableFile);
	writeString(methodName, variableFile); // TODO not required
	writeReference(dClassPos, variableFile);

	getLocVars(info->method, info->location, thread, depth);

	printDepth--;
	(*env)->Deallocate(env, (unsigned char*)methodName);
	return position;

}

FPOS_T getThreadInfo(jthread * thread) {
	int i;
	int maxDepth = 0;
	jvmtiThreadInfo info;
	jvmtiFrameInfo  * frames;
	jvmtiError err;

	jint ownedMonitorCount;
	jobject * ownedObjects = NULL;

	jobject contendedObject;
	struct JObject * monObj;

	jint frameCount;
    jint threadState;

	FPOS_T position;
	FPOS_T refLoc;
	FPOS_T tempRef;
	if (maxFrameDepth != 0){
		maxDepth = maxFrameDepth;
		frames = malloc(sizeof (jvmtiFrameInfo) * maxDepth);
	}else{
		maxDepth = 20;
		frames = malloc(sizeof (jvmtiFrameInfo) * maxDepth);
	}
	printDepth++;
	info.name = NULL;
	err = (*env)->GetThreadInfo(env, *thread, &info);
	if (err != JVMTI_ERROR_NONE) {
		(*env)->Deallocate(env, (unsigned char*)info.name);
		printf("No thread info %d \n ", err);
		abort();
	}
	err = (*env)->GetCurrentContendedMonitor(env, *thread, &contendedObject);
	if (err != JVMTI_ERROR_NONE) {
		contendedObject = 0;
	}
	err = (*env)->GetOwnedMonitorInfo(env, *thread, &ownedMonitorCount,
			&ownedObjects);
	if (err != JVMTI_ERROR_NONE) {
		ownedMonitorCount = 0;
	}

    err = (*env)->GetThreadState(env, *thread, &threadState);
    if (err != JVMTI_ERROR_NONE) {
        threadState = 0;
    }

	err = (*env)->GetStackTrace(env, *thread, 0, maxDepth, frames, &frameCount);
		if (err != JVMTI_ERROR_NONE) {
			frameCount = 0;
		}
	while (maxFrameDepth == 0 && frameCount == maxDepth ){
		printf("Had to double frame buffer, recommend limiting stack depth current buffer size: %d\n", maxDepth *2);
		free(frames);
		maxDepth = maxDepth * 2;
		frames = malloc(sizeof (jvmtiFrameInfo) * maxDepth);
		err = (*env)->GetStackTrace(env, *thread, 0, maxDepth, frames, &frameCount);
		if (err != JVMTI_ERROR_NONE) {
			frameCount = 0;
		}
	}
	printf("Thread %s Number of frames %d\n", info.name, frameCount);


	// Prepare space in dump file for all information queried 
	position = FTELL(variableFile);
	writeIDNoSize(CJVMTI_THREAD, variableFile);
	writeString(info.name, variableFile);
	(*env)->Deallocate(env, (unsigned char*)info.name);
	info.name = NULL;
	writeReference(addObject(*thread, 0)->positionInFile, variableFile);
	writeJint(info.priority, variableFile);
	writeBool(info.is_daemon, variableFile);
    writeJint(threadState, variableFile);
	writeJint(ownedMonitorCount, variableFile);
	refLoc = FTELL(variableFile);
	for (i = 0; i < ownedMonitorCount; i++) {
		writeReference(waitingToBeWritten, variableFile); // Owned monitor objects
	}
	writeReference(waitingToBeWritten, variableFile); // Contended object
	writeIDNoSize(CJVMTI_FRAME, variableFile);
	writeJint(frameCount, variableFile);
	for (i = 0; i < frameCount; i++) {
		writeReference(waitingToBeWritten, variableFile); // Stack frames
	}
	
	// Allocated space, now follow references
	FSEEK(variableFile, 0, SEEK_END);
	for (i = 0; i < ownedMonitorCount; i++) {

		monObj = addObject(ownedObjects[i], 0);

		FSEEK(variableFile, refLoc, SEEK_SET);
		writeReference(monObj->positionInFile, variableFile);
		refLoc = FTELL(variableFile);
		FSEEK(variableFile, 0, SEEK_END);
	}
	(*env)->Deallocate(env, (unsigned char*)ownedObjects);
	ownedObjects = NULL;
	if (contendedObject != 0) {

		monObj = addObject(contendedObject, 0);

		FSEEK(variableFile, refLoc, SEEK_SET);
		writeReference(monObj->positionInFile, variableFile);
	} else {
		FSEEK(variableFile, refLoc, SEEK_SET);
		writeReference(CJVMTI_NULL_OBJECT, variableFile);
	}
	writeIDNoSize(CJVMTI_FRAME, variableFile);
	writeJint(frameCount, variableFile);
	refLoc = FTELL(variableFile);
	FSEEK(variableFile, 0, SEEK_END);
	for (i = 0; i < frameCount; i++) {

		tempRef = getFrameInfo(i, &frames[i], thread);

		FSEEK(variableFile, refLoc, SEEK_SET);
		writeReference(tempRef, variableFile);
		refLoc = FTELL(variableFile);
		FSEEK(variableFile, 0, SEEK_END);
	}
	free(frames);
	return position;
}

int followAllThreads() {
	jvmtiError err;
	jint threadCount;
	jthread * threads;
	FPOS_T refLoc;
	FPOS_T ref;
	FPOS_T check;
	int i;

	err = (*env)->GetAllThreads(env, &threadCount, &threads);

	// Prepare space in file
	writeIDNoSize(CJVMTI_THREAD, variableFile);
	writeJint(threadCount, variableFile);
	refLoc = FTELL(variableFile);
	for (i = 0; i < threadCount; i++) {
		writeReference(waitingToBeWritten, variableFile); 
	}
	printf("%d threads \n", threadCount);
	check = FTELL(variableFile);
	for (i = 0; i < threadCount; i++) {
		//clearObjectQueue();
		addObject(threads[i], 0); // Add all thread objects
	}


	// Now fill in all references
	FSEEK(variableFile, 0, SEEK_END);
	for (i = 0; i < threadCount; i++) {

		ref = getThreadInfo(&threads[i]);

		FSEEK(variableFile, refLoc, SEEK_SET);
		writeReference(ref, variableFile);
		refLoc = FTELL(variableFile);
		FSEEK(variableFile, 0, SEEK_END);
	}

	clearObjectQueue();
	(*env)->Deallocate(env, (unsigned char*)threads);
	return 1;
}
