/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.harness;

import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.apache.kato.tck.api.TCKException;


public class MapCollector {

	private Map variables=new TreeMap();

	/**
	 * Create collector with provided  initial values
	 * 
	 * @param environmentalVariables
	 */
	public MapCollector(List environmentalVariables) {
		Iterator i=environmentalVariables.iterator();
		while(i.hasNext()) {
			String value=(String) i.next();
			collect(value,null);
		}
	}
	public MapCollector() {
		
	}


	/**
	 * Collects  variable key/value settings 
	 * 
	 * If a method registers a conflicting requirement
	 * a TCK exception is thrown.
	 *  
	 * @param var - variable to record  (assumes that key and value are 
	 *              seperated by an '='
	 * @param method - method doing the recording
	 * 
	 */
	public void collect(String var, Method method) {

		
		String key=null;
		String value=null;
		
		String[] separateVar=splitEnvVar(var);
		
		
		collect(separateVar[0],separateVar[1],method);


	}

	public static String[] splitEnvVar(String var) {
		
		String key=null;
		String value=null;
		int eq=var.indexOf('=');

		if(eq<0) {
			key=var;
		}
		else {
			key=var.substring(0,eq);
			value=var.substring(eq+1);
			
		}
		return new String[]{key,value};

	}
	public void collect(String key, String value, Method method) {

		if(variables.containsKey(key)==false) {
			CollectionValue v=new CollectionValue();
			v.recorder=method;
			v.value=value;
			variables.put(key,v);
		}
		else {
			// key clash
			CollectionValue original=(CollectionValue) variables.get(key);

			if(original.value==null) {
				if( value==null) return;  // matching ultimate value (both null)
			}
			else {
				if( value!=null) {
					// both not null.  same value?
					if(original.equals(value)) return; // same value
				}
			}

			String originator="<intial setup>";
			if(original.recorder!=null) originator=original.recorder.toString();
			throw new TCKException("unable to register key "+key+" for "+method+" value conflicts with previously registered value from "+originator);
		}
	}

	class CollectionValue {
		Method recorder=null;
		String value=null;
	}

	public List asList() {

		List list=new LinkedList();
		Iterator i=variables.keySet().iterator();
		while(i.hasNext()) {
			Object key=i.next();
			CollectionValue value=(CollectionValue) variables.get(key);
			String keyValue=key.toString();
			if(value.value==null) list.add(keyValue);
			else list.add(keyValue+"="+value.value);
		}

		return list;
	}

	public Iterator keyIterator() {
		return variables.keySet().iterator();
	}

	public String getValue(String key) {
		CollectionValue value=(CollectionValue) variables.get(key);
		if(value==null)  return null;
		return value.value;
	}

	public int size() {
		return variables.size();
	}

	
}
