/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.results.reporter;

import java.io.File;
import java.io.IOException;
import java.net.URL;

public class TCKConsoleReportsAggregator {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		
		File out=new File(args[0]);
		out.mkdirs();
		
		ConsoleOutputResultsReader reader=new ConsoleOutputResultsReader();
		
		TCKResultMatrix mainMatrix=new TCKResultMatrix(new FullSortedProvider());
		
		System.out.println("output to "+out.getAbsolutePath());
		int returnCode=0;
		
		for(int i=1;i<args.length;i++) {
			String parm=args[i];
			try {
				System.out.println("reading "+parm);
				URL u=new URL(parm);
				TCKResultMatrix matrix=reader.getResultSet(u);
				System.out.println("Resulting matrix has "+matrix.rowCount()+" testcases * "+matrix.columnCount()+" result sets ");
				mainMatrix.addMatrix(matrix);
				
			}  catch (IOException e) {
				e.printStackTrace();
				returnCode=-1;
			}
			 
		}
		
		System.out.println("Final tally: "+mainMatrix.rowCount()+" testcases * "+mainMatrix.columnCount());
				
		
		TCKResultMatrixHTMLWriter writer=new TCKResultMatrixHTMLWriter(out);
		writer.setVerbose(true);
		
		try {
			writer.print(mainMatrix);
		} catch (IOException e) {

			e.printStackTrace();
			returnCode=-2;
		}

		System.exit(returnCode);
	}

}
