/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package test.apache.kato.tck.harness.unittests;

import java.util.Properties;

import org.apache.kato.tck.api.PropertyStore;

import junit.framework.TestCase;


public class TestPropertyStore extends TestCase {

	
	public void testLoadPropertyIntoTarget() {
		
		
		Properties source=new Properties();
		source.setProperty("a", "1");
		
		PropertyStore targetStore=new PropertyStore();
		PropertyStore sourceStore=new PropertyStore(source,"a",targetStore);
		
		assertTrue(sourceStore.hasProperty("a"));
		assertEquals("1",(String)sourceStore.getProperty("a"));
		
		
	}
	
	public void testNotOveridingIntoTarget() {
		
		PropertyStore target=new PropertyStore();
		PropertyStore source=new PropertyStore(target);
		
		
		target.setProperty("a", "1");
		source.setProperty("a", "2");
		
		
		assertTrue(target.hasProperty("a"));
		assertEquals("1",(String)target.getProperty("a"));
		
		
	}
	public void testCompleteReplacement() {
		
		PropertyStore target=new PropertyStore();
		PropertyStore source=new PropertyStore(target);
		
		target.setProperty("a", "1");
		source.setProperty("b", "${a}");
		
		
		assertTrue(source.hasProperty("b"));
		assertEquals("1",(String)source.getProperty("b"));
		
		
	}
	public void testMultipleReplacement() {
		
		PropertyStore target=new PropertyStore();
		PropertyStore source=new PropertyStore(target);
		
		
		target.setProperty("a", "1000");
		source.setProperty("b", "start ${a} middle ${a} end");
		
		
		assertTrue(source.hasProperty("b"));
		assertEquals("start 1000 middle 1000 end",(String)source.getProperty("b"));
		
		
	}
	public void testEndReplacement() {
		
		PropertyStore target=new PropertyStore();
		PropertyStore source=new PropertyStore(target);
		
		target.setProperty("end", "1000");
		source.setProperty("b", "start ${end}");
		
		
		
		assertTrue(source.hasProperty("b"));
		assertEquals("start 1000",(String)source.getProperty("b"));
		
		
	}
	public void testStartReplacement() {
		
		PropertyStore target=new PropertyStore();
		PropertyStore source=new PropertyStore(target);
		
		target.setProperty("start", "1000");
		source.setProperty("b", "${start} end");
		
		
		
		assertTrue(source.hasProperty("b"));
		assertEquals("1000 end",(String)source.getProperty("b"));
		
		
	}
	
	public void IgnoretestCyclicReferencesDetected() {
	
		PropertyStore a=new PropertyStore();
		a.setProperty("a", "${b}");
		PropertyStore b=new PropertyStore(a);
		b.setProperty("b", "${a}");
		try {
		String value=b.getProperty("b");
		fail("expected runtime exception");
		}
		catch(RuntimeException re) {
			; // as expected
		}
		
		
		
	}
	public void testDynamicUpdates() {
		
		PropertyStore a=new PropertyStore();
		a.setProperty("a", "1");
		PropertyStore b=new PropertyStore(a);
		
		b.setProperty("b", "${a}");
		
		assertEquals("1",b.getProperty("b"));
		
		// change property
		a.setProperty("a", "2");
		assertEquals("2",b.getProperty("b"));
		
	}

	public void testPropertyAsArrayNoValue() {

		PropertyStore a=new PropertyStore();
		
		String[] array=a.getPropertyAsArray("a",",");
		assertNull(array);
		
	}

	public void testPropertyAsArrayNoSeperator() {

		PropertyStore a=new PropertyStore();
		a.setProperty("a", "123456");
		String[] array=a.getPropertyAsArray("a",",");
		assertEquals("123456",array[0]);
		
	}
	public void testPropertyAsArraySeperatorAtFront() {

		PropertyStore a=new PropertyStore();
		a.setProperty("a", ",123456");
		String[] array=a.getPropertyAsArray("a",",");
		assertEquals("",array[0]);
		assertEquals("123456",array[1]);
		
	}
	public void testPropertyAsArraySeperatorAtEnd() {

		PropertyStore a=new PropertyStore();
		a.setProperty("a", "123456,");
		String[] array=a.getPropertyAsArray("a",",");
		
		assertEquals("123456",array[0]);
		assertEquals("",array[1]);
		
	}
	
	
	public void testPropertyAsArraySimpleArray() {

		PropertyStore a=new PropertyStore();
		a.setProperty("a", "1,2,3");
		
		String[] array=a.getPropertyAsArray("a",",");
		assertNotNull(array);
		assertEquals(3,array.length);
		assertEquals("1",array[0]);
		assertEquals("2",array[1]);
		assertEquals("3",array[2]);
	}
	
	public void testNestedProperties() {
	
		PropertyStore a=new PropertyStore();
		a.setProperty("a", "123");
		a.setProperty("b", "${a}");
	
		String result=a.getProperty("b");
		assertEquals("123",result);
	}
	
}
