/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import org.apache.kato.tck.api.ICheckpoint;
import org.apache.kato.tck.harness.scenario.ScenarioElement;


public class SetupJavaRuntime_getHeapRoots extends ScenarioElement  {
	
	public static final String DTFJ_STRING_TABLE_ROOT_TEST = "DTFJ StringTable root test";


	public static java.lang.ClassLoader cl = null;
	
	public HeapRootsTestUnfinalized unfinalizedObj = null;
	
	public static class HeapRoot {
		private Object root = null;
		private int type = 0;
		
		public HeapRoot(Object r, int t) {
			root = r;
			type = t;
		}
		
		public boolean equals(Object o) {
			if (! (o instanceof HeapRoot)) {
				return false;
			} else {
				HeapRoot ro = (HeapRoot) o;
				return root.equals(ro.getRoot()) && type==0?true:type==ro.getType(); 
			}
		}
		
		public int hashCode() {
			return root.hashCode();
		}
		
		public Object getRoot() {
			return root;
		}

		public int getType() {
			return type;
		}
		
		
	
	}
	
	class HeapRootsTestClassLoader extends ClassLoader {
		
		public Class findClass(String name) {
			byte[] b = loadClassData(name);
			return defineClass(name, b, 0, b.length);
		}
		private byte[] loadClassData(String name) {
			return null;
		}
	}

	class HeapRootsTestThread extends java.lang.Thread {
		public void run() {
			while (true) {
				try {
					sleep(10000);
				} catch (InterruptedException e) {}
			}
		}
	}

	
	class HeapRootsTestFinalizable {
		protected void finalize() {
//			System.out.println("Finalizer for object " + this + " started in thread " + Thread.currentThread());
			while (true) {
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
	}

	class HeapRootsTestUnfinalized {
		protected void finalize() {
			System.out.println("Finalizer for object " + this + " started...");
		}
	}

	
	public void configureClassloaderRoot() {

		cl = new HeapRootsTestClassLoader();
	}
	
	public void configureUnfinalizedRoot(ICheckpoint cp) {
		unfinalizedObj = new HeapRootsTestUnfinalized();
		cp.checkpoint();
		System.out.println(unfinalizedObj);
	}

	public void configureFinalizableRoot() {
		HeapRootsTestFinalizable finalizableObj = new HeapRootsTestFinalizable();
		HeapRootsTestFinalizable finalizableObj2 = new HeapRootsTestFinalizable();
		finalizableObj = null;
		finalizableObj2 = null;
		System.gc();
	}

	public void configureStringTableRoot() {
		String string = DTFJ_STRING_TABLE_ROOT_TEST;
		string.intern();
	}
	
	public void configureThreadRoot(ICheckpoint cp) {
		HeapRootsTestThread thread = new HeapRootsTestThread();
		thread.start();
		cp.checkpoint();
	}
	
	class HeapRootsTestParam {}
	class HeapRootsTestLocalVariable {}

	public final String javaLocalRootsthreadName = this.getClass().getName()+"#configureJavaLocalsRoots";
	public void configureJavaLocalsRoots(ICheckpoint cp) {
		HeapRootsTestParam param = new HeapRootsTestParam();
		methodA(cp, param);
	}
	
	private void methodA(ICheckpoint cp, HeapRootsTestParam param) {
		HeapRootsTestLocalVariable localVariable = new HeapRootsTestLocalVariable();
		cp.checkpoint();
		System.out.println(param);
		System.out.println(localVariable);
	}
	
	
	
	

}
