/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.tck.tests.legacy;

import java.util.List;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.image.ImageSection;
import javax.tools.diagnostics.image.ImageThread;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaRuntime;
import javax.tools.diagnostics.runtime.java.JavaStackFrame;
import javax.tools.diagnostics.runtime.java.JavaThread;

import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;

/**
 * 
 */
public class TestJavaThread  extends TCKJavaRuntimeTestcase {

	
	public void testThreads() {
		JavaRuntime runtime=getJavaRuntime();
		List<JavaThread> threads=runtime.getThreads();
		if(threads.isEmpty()) return; // no threads to test
		
		int counter=0;
		
		for(JavaThread t:threads) {
			
			
			try {
				
				ImageThread ithread=t.getImageThread();
				assertNotNull("Entry "+counter+":image thread is null - DataUnavailable should have been thrown instead",ithread);
				
			} catch (CorruptDataException e) {
				// allowed
			} catch (DataUnavailable e) {
				// allowed
			}
		
			try {
				ImagePointer p=t.getJNIEnv();
				assertNotNull("Entry "+counter+":JNIEnv is null - DataUnavailable should have been thrown instead",p);
			} catch (CorruptDataException e) {
				// allowed
			}
			
			
			try {
				String name=t.getName();
				assertNotNull("Entry "+counter+":name is null - DataUnavailable should have been thrown instead",name);
				
			} catch (CorruptDataException e) {
				// allowed
			}
			
			try {
				JavaObject obj=t.getObject();
				assertNotNull("Entry "+counter+":object is null - DataUnavailable should have been thrown instead",obj);
			} catch (CorruptDataException e) {
			// allowed
			}
			
			try {
				int priority=t.getPriority();
				assertTrue("Entry "+counter+":priority value ["+priority+"] is not >=1 && <=10",priority>=1 && priority<=10);
			} catch (CorruptDataException e) {
				// allowed;
			}
			
			List<JavaStackFrame> frames=t.getStackFrames();
			assertNotNull("Entry "+counter+":frames list is  null",frames);
			
			List<ImageSection> sections=t.getStackSections();
			assertNotNull("Entry "+counter+":section list is  null",sections);
			
			try {
				int state=t.getState();
				
			} catch (CorruptDataException e) {
				// allowed
			}
		
			counter++;
		}
		
		
		
	}
}
