/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import java.util.Iterator;
import java.util.List;

import javax.tools.diagnostics.image.CorruptData;
import javax.tools.diagnostics.image.ImageAddressSpace;
import javax.tools.diagnostics.image.ImageSection;


public class ImageAddressSpaceTest extends AbstractImageTestcase {
	private ImageAddressSpace _space = null;
	public static final long TEST_ADDRESS = 0x7FFFFFFF;	//TODO: don't test with this specific value
	
	protected void setUp() throws Exception {
		_space = defaultAddressSpace();
		super.setUp();
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageAddressSpace.getCurrentProcess()'
	 * 
	 * Ensures that the process is non-null
	 */
	public void testGetCurrentProcess() {
		assertNotNull(_space.getCurrentProcess());
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageAddressSpace.getProcesses()'
	 * 
	 * Ensures that the process iterator is non-null and has at least one process
	 */
	public void testGetProcesses() {
		List it = _space.getProcesses();
		
		assertNotNull(it);
		assertFalse(it.isEmpty());
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageAddressSpace.getPointer(long)'
	 * 
	 * Ensures that pointers can be created which point at entities within the address space.
	 * TODO:  this needs to be tested in a less arbitrary way.
	 */
	public void testGetPointer() {
		assertNotNull(_space.getPointer(TEST_ADDRESS));
	}
	
	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageAddressSpace.getImageSections()'
	 * 
	 * Ensures that there is at least one section and it is of type ImageSection
	 */
	public void testGetImageSections()
	{
		Iterator it = _space.getImageSections().iterator();
		assertTrue(it.hasNext());
		// empty iterator is a valid return from getImageSections
		while (it.hasNext()) {
			Object section = it.next();
			assertTrue((section instanceof ImageSection) || (section instanceof CorruptData));
		}
	}
}
