/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImageAddressSpace;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.image.MemoryAccessException;

public class ImagePointerTest extends AbstractImageTestcase
{
	private static ImageAddressSpace _parent;	//used for comparisons
	private static long _address;	//used for comparisons
	private static final long OFFSET = 0x100;
	private ImagePointer _testPointer;
	
	public  ImagePointer defaultImagePointer()
	{
		_parent = defaultAddressSpace();
		_address = ImageAddressSpaceTest.TEST_ADDRESS;
		
		return _parent.getPointer(_address);
	}
	
	protected void setUp() throws Exception
	{
		_testPointer = defaultImagePointer();
		super.setUp();
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.getAddress()'
	 * 
	 * Ensure that the address is what we created it with
	 */
	public void testGetAddress()
	{
		assertTrue(_address == _testPointer.getAddress());
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.getAddressSpace()'
	 * 
	 * Ensures that the parent address space is equal to the one we used to create it
	 */
	public void testGetAddressSpace()
	{
		assertTrue(_parent.equals(_testPointer.getAddressSpace()));
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.add(long)'
	 * 
	 * Ensures that adding an offset yields a pointer at the expected address
	 */
	public void testAdd()
	{
		ImagePointer newPointer = _testPointer.add(OFFSET);
		assertTrue(newPointer.getAddress() == (_testPointer.getAddress() + OFFSET));
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.isExecutable()'
	 * 
	 * Ensures that the method can be called
	 */
	public void testIsExecutable()
	{
		try {
			_testPointer.isExecutable();
		} catch (DataUnavailable e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.isReadOnly()'
	 * 
	 * Ensures that the method can be called
	 */
	public void testIsReadOnly()
	{
		try {
			_testPointer.isReadOnly();
		} catch (DataUnavailable e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.isShared()'
	 * 
	 * Ensures that the method can be called
	 */
	public void testIsShared()
	{
		try {
			_testPointer.isShared();
		} catch (DataUnavailable e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.getPointerAt(long)'
	 * 
	 * Ensure that we can read a pointer at the given address without throwing anything unexpected
	 */
	public void testGetPointerAt()
	{
		try {
			_testPointer.getPointerAt(0);
		} catch (MemoryAccessException e) {
			assertNotNull(e.getPointer());
			//accepted by spec
		} catch (CorruptDataException e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.getLongAt(long)'
	 * 
	 * Ensure that we can read a long at the given address without throwing anything unexpected
	 */
	public void testGetLongAt()
	{
		try {
			_testPointer.getLongAt(0);
		} catch (MemoryAccessException e) {
			assertNotNull(e.getPointer());
			//accepted by spec
		} catch (CorruptDataException e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.getIntAt(long)'
	 * 
	 * Ensure that we can read an int at the given address without throwing anything unexpected
	 */
	public void testGetIntAt()
	{
		try {
			_testPointer.getIntAt(0);
		} catch (MemoryAccessException e) {
			assertNotNull(e.getPointer());
			//accepted by spec
		} catch (CorruptDataException e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.getShortAt(long)'
	 * 
	 * Ensure that we can read a short at the given address without throwing anything unexpected
	 */
	public void testGetShortAt()
	{
		try {
			_testPointer.getShortAt(0);
		} catch (MemoryAccessException e) {
			assertNotNull(e.getPointer());
			//accepted by spec
		} catch (CorruptDataException e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.getByteAt(long)'
	 * 
	 * Ensure that we can read a byte at the given address without throwing anything unexpected
	 */
	public void testGetByteAt()
	{
		try {
			_testPointer.getByteAt(0);
		} catch (MemoryAccessException e) {
			assertNotNull(e.getPointer());
			//accepted by spec
		} catch (CorruptDataException e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.getFloatAt(long)'
	 * 
	 * Ensure that we can read a float at the given address without throwing anything unexpected
	 */
	public void testGetFloatAt()
	{
		try {
			_testPointer.getFloatAt(0);
		} catch (MemoryAccessException e) {
			assertNotNull(e.getPointer());
			//accepted by spec
		} catch (CorruptDataException e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImagePointer.getDoubleAt(long)'
	 * 
	 * Ensure that we can read a double at the given address without throwing anything unexpected
	 */
	public void testGetDoubleAt()
	{
		try {
			_testPointer.getDoubleAt(0);
		} catch (MemoryAccessException e) {
			assertNotNull(e.getPointer());
			//accepted by spec
		} catch (CorruptDataException e) {
			//accepted by spec
		}
	}
	
	/**
	 * Verify that the equals call doesn't throw
	 */
	public void testEquals()
	{
		try{
			assertTrue(_testPointer.equals(_testPointer));
		} catch (Throwable t) {
			assertTrue(false);
		}
	}
	
	/**
	 * Verify that hashCode() doesn't throw and returns non-zero (technically zero is ok but it will be 
	 * flagged here to ensure that we aren't doing anything bad to create the hashcode)
	 */
	public void testHashCode()
	{
		try {
			assertTrue(0 != _testPointer.hashCode());
		} catch (Throwable t) {
			assertTrue(false);
		}
	}
}
