/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import java.util.Iterator;

import javax.tools.diagnostics.image.CorruptData;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImageRegister;
import javax.tools.diagnostics.image.ImageSection;
import javax.tools.diagnostics.image.ImageStackFrame;
import javax.tools.diagnostics.image.ImageThread;


public class ImageThreadTest extends AbstractImageTestcase
{
	private ImageThread _thread = null;
	
	protected void setUp() throws Exception
	{
		_thread = defaultImageThread();
		assertNotNull(_thread);
		super.setUp();
	}

	/*
	 * Test method for 'javax.tools.diagnostics.image.ImageThread.getID()'
	 */
	public void testGetID()
	{
		try {
			String id = _thread.getID();
			assertNotNull(id);
			Long.decode(id);
		} catch (Exception e) {
			fail();
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageThread.getStackFrames()'
	 * 
	 * Ensures that a non-null iterator is returned with at lzero or more elements
	 * of type ImageStackFrame or CorruptData.
	 * 
	 */
	public void testGetStackFrames()
	{
		try {
			Iterator frames = _thread.getStackFrames().iterator();
			
			assertNotNull(frames);
			while (frames.hasNext()) {
				Object frame = frames.next();
				assertTrue((frame instanceof ImageStackFrame) || (frame instanceof CorruptData));
			}
		} catch (DataUnavailable e) {
			//acceptable in spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageThread.getStackSections()'
	 * 
	 * Ensures that at least one stack section exists
	 */
	public void testGetStackSections()
	{
		Iterator it = _thread.getStackSections().iterator();
		assertNotNull(it);
		// empty iterator is a valid return from getStackFrames
		while (it.hasNext()) {
			Object section = it.next();
			assertTrue((section instanceof ImageSection) || (section instanceof CorruptData));
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageThread.getRegisters()'
	 * 
	 * Ensures that a non-null, non-empty iterator is returned and all instances
	 * are of type ImageRegister
	 */
	public void testGetRegisters()
	{
		Iterator regs = _thread.getRegisters().iterator();
		
		assertNotNull(regs);
		// Empty iterator is a valid return from getRegisters
		while (regs.hasNext()) {
			Object reg = regs.next();
			// if there are registers check they are valid
			assertTrue(reg instanceof ImageRegister);
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageThread.getProperties()'
	 * 
	 * Ensures that non-null properties are returned
	 */
	public void testGetProperties()
	{
		assertNotNull(_thread.getProperties());
	}
}
