/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import javax.tools.diagnostics.image.ImageFactory;

import junit.framework.TestCase;


/**
 * Simple tests to show that the manifest entries that describe the 
 * implementation and specification for DTFJ have made their way 
 * into the relevant Package instance.
 * 
 *
 */
public class TestPackageInformation extends TestCase {

	/**
	 * Test that the specification title is what is expected
	 */
	
	public void testSpecificationTitle() {

		Package p = ImageFactory.class.getPackage();
		assertEquals("Diagnostic Tool Framework for Java", p
				.getSpecificationTitle());
	}

	/**
	 * Test that the specification version matches  
	 */
	public void testSpecificationVersion() {

		String versionString=ImageFactory.MAJOR_VERSION+"."+ImageFactory.MINOR_VERSION;
		Package p = ImageFactory.class.getPackage();
		assertEquals(versionString, p.getSpecificationVersion());
	}

	
	/**
	 * Test Implementation version is present 
	 */
	
	public void testImplementationVersion() {

		Package p = ImageFactory.class.getPackage();
		assertNotNull(p.getImplementationVersion());
	}

	/**
	 * Test implementation title is present
	 */
	public void testImplementationTitle() {
		Package p = ImageFactory.class.getPackage();
		assertNotNull(p.getImplementationTitle());

	}

}
