/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview;

public class ConsoleOutputChannel implements OutputChannel {
	
	private boolean noPrint = false;

	public void print(String outputString) {
		if (!noPrint)
			System.out.print(outputString);
	}

	public void printPrompt(String prompt) {
		System.out.print(prompt);
	}

	public void println(String outputString) {
		if (!noPrint)
			System.out.println(outputString);
	}

	public void error(String outputString) {
		System.err.print("\n");
		System.err.print("\tERROR: " + outputString + "\n");
	}

	public void printInput(long timestamp, String prompt, String outputString) {
		// we don't need to output anything, but we could output the time the command was started
		//System.out.println("<started at: " + (new Date(timestamp)).toString() + ">");
	}
	
	public void close() {
		// do nothing, because we don't need to close System.out or System.err
	}
	
	public void setNoPrint(boolean b) {
		noPrint = b;
	}
}
