/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands.infocommands;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.Image;
import javax.tools.diagnostics.runtime.java.JavaClass;
import javax.tools.diagnostics.runtime.java.JavaMonitor;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaRuntime;
import javax.tools.diagnostics.runtime.java.JavaThread;

import org.apache.kato.katoview.Output;
import org.apache.kato.katoview.commands.Command;
import org.apache.kato.katoview.commands.helpers.Exceptions;
import org.apache.kato.katoview.commands.helpers.Utils;


public class InfoLockCommand extends Command{
	
	public InfoLockCommand(Output o){
		super(o, "lock", "outputs a list of system monitors and locked objects",
				"parameters: none\n\n" +
				"The info lock command outputs a list of system monitors and locked objects.");
		child_commands = null;
	}
	
	public void doCommand(Stack args, Image loadedImage, HashMap properties){
		Vector vMonitorsWithLockedObjects = new Vector();
		Iterator runtimes = Utils.getRuntimes(loadedImage);
		
		while(runtimes.hasNext()){
			JavaRuntime jRuntime = (JavaRuntime)runtimes.next();
			Iterator monitors = jRuntime.getMonitors().iterator();
			
			out.println("\nSystem locks...");
			while (monitors.hasNext()){
				JavaMonitor jMonitor = (JavaMonitor)monitors.next();
				JavaObject jObject = jMonitor.getObject();
				try {
					String monitorName = jMonitor.getName().trim();
					if (monitorName.equalsIgnoreCase("")) {
						monitorName = "<un-named monitor>";
					}
					out.println("id: 0x" + jMonitor.getID() + " name: " + jMonitor.getName());

					JavaThread owner = jMonitor.getOwner();
					if (null != owner) {
						try {
							out.println("\towner thread id: " + owner.getImageThread().getID()
									+ " name: " + owner.getName());
						} catch (DataUnavailable e) {
							out.println("\towner thread id: " + Exceptions.getDataUnavailableString());
						}
					}
					
					// List any threads waiting on enter or notify for this monitor
					Iterator itEnterWaiter = jMonitor.getEnterWaiters().iterator();
					while (itEnterWaiter.hasNext()) {
						JavaThread jThread = (JavaThread)itEnterWaiter.next();
						try {
							out.println("\twaiting thread id: " + jThread.getImageThread().getID() 
									+ " name: " + jThread.getName());
						} catch (DataUnavailable dae) {
							out.println("\twaiting thread id: <unknown>");
						}
					}
					Iterator itNotifyWaiter = jMonitor.getNotifyWaiters().iterator();
					while (itNotifyWaiter.hasNext()) {
						JavaThread jThread = (JavaThread)itNotifyWaiter.next();
						try {
							out.println("\twaiting thread id: " + jThread.getImageThread().getID() 
									+ " name: " + jThread.getName());
						} catch (DataUnavailable dae) {
							out.println("\twaiting thread id: <unknown>");
						}
					}
					
				} catch(CorruptDataException cde) {
					out.println("\nwarning, corrupt data encountered during scan for system locks...");
				}
				if (null != jObject) {
					// Remember object monitors (flat or inflated) for later
					vMonitorsWithLockedObjects.add(jMonitor);
				}
			}
		}
		
		out.println("\nLocked objects...");
		if (0 == vMonitorsWithLockedObjects.size()){
			out.println("\t...None.");
			return;
		}
		Iterator lockedObjects = vMonitorsWithLockedObjects.iterator();
		while(lockedObjects.hasNext()){
			JavaMonitor jMonitor = (JavaMonitor)lockedObjects.next();
			JavaObject jObject = jMonitor.getObject();
			try{
				JavaThread owner = jMonitor.getOwner();
				String className = "<unknown class>";
				JavaClass jClass = jObject.getJavaClass();	
				if (null != jClass) {
				    className = jClass.getName();
				}
				String jObjectID = Long.toHexString(jObject.getID().getAddress());
				if (null == owner) {
					out.println(className + "@0x" + jObjectID + " locked by an unknown thread");
				} else {
					String owningThreadID = null;
					try {
						owningThreadID = owner.getImageThread().getID();
					} catch (DataUnavailable e) {
						owningThreadID = Exceptions.getDataUnavailableString();
					}
					out.println(className + "@0x" + jObjectID + "\n\tlocked by thread id: "
								+ owningThreadID + " name: " + owner.getName());
				}
			}catch(CorruptDataException cde){
			}
		}
	}
}
