/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands.showcommands;

import java.util.HashMap;
import java.util.Stack;

import javax.tools.diagnostics.image.Image;

import org.apache.kato.katoview.Output;
import org.apache.kato.katoview.commands.Command;
import org.apache.kato.katoview.heapdump.HeapDumpSettings;


public class ShowHeapdumpCommand extends Command
{
	public static final String COMMAND_NAME = "heapdump";
	public static final String COMMAND_DESCRIPTION = "displays heapdump settings";
	public static final String LONG_DESCRIPTION = "Parameters:none\n\n"
		+ "Prints heapdump format and file name.\n"
		+ "Use \"set heapdump\" to change settings\n";

	public ShowHeapdumpCommand(Output o) {
		super(o,COMMAND_NAME,COMMAND_DESCRIPTION,LONG_DESCRIPTION);
	}

	public void doCommand(Stack args, Image loadedImage, HashMap properties)
	{
		if(! args.isEmpty()) {
			out.error("\"show heapdump\" does not take any parameters.");
			return;
		}
		
		out.print("Heapdump Settings:\n\n");
		
		out.print("\tFormat: " + (HeapDumpSettings.areHeapDumpsPHD(properties) ? "PHD" : "Classic (TXT)") + "\n");
		out.print("\tFile Name: " + HeapDumpSettings.getFileName(properties) + "\n");
		out.print("\tMultiple heaps will be written to " 
				+ (HeapDumpSettings.multipleHeapsInMultipleFiles(properties) ? "multiple files":"a single file") 
				+ "\n");
	}
	
}
