/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands.showcommands;

import java.util.HashMap;
import java.util.Stack;

import javax.tools.diagnostics.image.Image;

import org.apache.kato.katoview.Output;
import org.apache.kato.katoview.commands.Command;


public class ShowLoggingCommand extends Command {

	public ShowLoggingCommand(Output o){
		super(o, "logging", "displays the current values of logging settings",
				"parameters: none\n\n" +
				"outputs following values, which are generally set by the \"set " +
				 "logging\" command:\n" +
				" set_logging = [on|off]\n" +
				" set_logging_file = <filename>\n" +
				" set_logging_overwrite = [on|off]\n" +
				" set_logging_redirect = [on|off]\n" +
				" current_logging_file = <filename> - file that is currently being " +
				 "logged to; could be different than set_logging_file if that value " +
				 "was changed after logging was started"
				);
		child_commands = null;
	}
	
	public void doCommand(Stack args, Image loadedImage, HashMap properties){
		if (!args.isEmpty())
		{
			out.error("\"show logging\" does not take any parameters");
			return;
		}
		
		String fileVal = (String)properties.get("set_logging_file");
		if (null == fileVal)
			fileVal = "<not set>";
		else
			fileVal = "\"" + fileVal + "\"";
		
		String currentFileVal = (String)properties.get("current_logging_file");
		if (null == currentFileVal)
			currentFileVal = "<not currently logging>";
		else
			currentFileVal = "\"" + currentFileVal + "\"";

		out.print("\n");
		printKeyVal(properties, "set_logging");
		out.println("\t" + "set_logging_file" + " = " + fileVal);
		printKeyVal(properties, "set_logging_overwrite");
		printKeyVal(properties, "set_logging_redirect");
		out.print("\n");
		out.println("\t" + "current_logging_file" + " = " + currentFileVal);
			
		out.print("\n");
	}
	
	private void printKeyVal(HashMap properties, String key)
	{
		out.println("\t" + key + " = " + getVal(properties, key));
	}
	
	private String getVal(HashMap properties, String key)
	{
		String val = (String)properties.get(key);

		if (null == val)
			return "off";
		else
			return val;
	}
}
